=== modified file 'CMakeLists.txt'
--- CMakeLists.txt	2015-10-25 19:56:39 +0000
+++ CMakeLists.txt	2015-10-28 14:45:22 +0000
@@ -32,9 +32,9 @@
     # This can not be tested using cross-compile methods with MingW and Wine.
     set (CMAKE_C_FLAGS "${CMAKE_C_FLAGS} -std=c11 -mthreads -Wall -gdwarf-2 -pthread")
 
-    set (CMAKE_C_FLAGS_DEBUG "${CMAKE_C_FLAGS_DEBUG} -std=c11 -mthreads -Wall -g -gdwarf-2 -O0")
+    set (CMAKE_C_FLAGS_DEBUG "${CMAKE_C_FLAGS_DEBUG} -std=c11 -mthreads -Wall -g -O0")
 
-    set (CMAKE_C_FLAGS_RELEASE "${CMAKE_C_FLAGS_RELEASE} -std=c11 -mthreads -Wall -gdwarf-2 ")
+    set (CMAKE_C_FLAGS_RELEASE "${CMAKE_C_FLAGS_RELEASE} -std=c11 -mthreads -Wall ")
   elseif (MSVC)
     message ("MSVC is not supported, Yet. Patches welcome!")
   endif ()

=== modified file 'libssts/Callback.c'
--- libssts/Callback.c	2015-04-09 18:45:12 +0000
+++ libssts/Callback.c	2015-10-28 14:45:22 +0000
@@ -22,4 +22,56 @@
 
 #include "Callback.h"
 
+SCallbackEntry *
+s_callback_entry_new_real (const schar * name,
+                           Callback callback,
+                           SCallbackType type) {
+  SCallbackEntry * self = malloc (sizeof (SCallbackEntry));
+  self->name = s_string_new (name);
+  self->callback = callback;
+  self->type = type;
+  return self;
+}
+
+void
+s_callback_entry_free (SCallbackEntry * self) {
+  //free (self->name)
+  free (self);
+}
+
+void
+s_object_install_callbacks (SObject * obj,
+                            size_t n_callbacks,
+                            SCallbackEntry ** callback_entries) {
+  assert (obj);
+  assert (callback_entries);
+  s_dbg_print ("Adding Callbacks, n_callbacks: %zd", n_callbacks);
+  for (size_t i = 0; i < n_callbacks ; i++) {
+    s_dbg_print ("Loop: Adding Callbacks");
+    SCallbackEntry * entry = callback_entries[i];
+    s_object_install_callback (obj, entry);
+  }
+}
+
+void
+s_object_install_callback (SObject * obj,
+                           SCallbackEntry * callback_entry) {
+  //
+  assert (obj);
+  assert (callback_entry);
+  s_dbg_print ("Adding callback");
+  s_map_add (obj->callbacks, callback_entry->name, callback_entry);
+}
+
+spointer
+s_object_notify (SObject * self,
+                 schar * name,
+                 spointer * user_data) {
+  //
+  SCallbackEntry * entry = s_map_get (self->callbacks, name);
+  assert (s_string_is_equal (entry->name, name));
+
+  Callback callback = entry->callback;
+  return callback (self, user_data);
+}
 

=== modified file 'libssts/Callback.h'
--- libssts/Callback.h	2015-09-16 11:31:00 +0000
+++ libssts/Callback.h	2015-10-28 14:45:22 +0000
@@ -35,7 +35,9 @@
 /**
  * A list containing the names of the callback types.
  */
-static char * SCallbackTypeNames[] __attribute__((unused)) = {
+UNUSED
+static char *
+SCallbackTypeNames[] = {
   "NULL",
   "CALLBACK",
   "NOTIFY",
@@ -56,7 +58,7 @@
 };
 s_object_install_callbacks (S_OBJECT(Foo), &foo_callback_entries);
  * @endcode
- * 
+ *
  * The callback must have the following signature:
  * @code{.c}
 spointer
@@ -67,12 +69,23 @@
  @endcode
  */
 typedef struct SCallbackEntry {
-  char * name; /**< The name that is used to invoke the callback. */
+  schar * name; /**< The name that is used to invoke the callback. */
   Callback callback; /**< The callback that will be invoked.*/
   SCallbackType type; /**< The type of the callback, is it a standard callback
                        * or a notify? */
 } SCallbackEntry;
 
+#define s_callback_entry_new(n, c, t)\
+ s_callback_entry_new_real (n, CALLBACK (c), t)
+
+SCallbackEntry *
+s_callback_entry_new_real (const schar * name,
+			   Callback callback,
+			   SCallbackType type);
+
+void
+s_callback_entry_free (SCallbackEntry * entry);
+
 /**
  * @}
  * @addtogroup SObject
@@ -81,12 +94,14 @@
 
 /**
  * Installs an array of callbackentries into an SObject.
- * 
+ *
  * @param obj The object to install the callback entries into.
  * @param callbackentries an array containing the callback entries.
  */
 void
-s_object_install_callbacks (SObject * obj, SCallbackEntry ** callcackentries);
+s_object_install_callbacks (SObject * obj,
+			    size_t n_callbacks,
+			    SCallbackEntry ** callback_entries);
 
 /**
  * Installs a singel callback into an SObject.
@@ -95,7 +110,7 @@
  * @param callbackentry The callback entry to install.
  */
 void
-s_object_install_callback (SObject * obj, SCallbackEntry * callbackentry);
+s_object_install_callback (SObject * obj, SCallbackEntry * callback_entry);
 
 /**
  * @breif Do a callback.

=== modified file 'libssts/DynamicArray.c'
--- libssts/DynamicArray.c	2015-10-26 18:59:48 +0000
+++ libssts/DynamicArray.c	2015-10-28 14:45:22 +0000
@@ -88,6 +88,7 @@
 s_dynamic_array_free (SDynamicArray * self, sboolean free_data) {
   if (free_data) {
     if (self->free_func != NULL) {
+      s_dbg_print ("SDynamicArray: free data.");
       s_dynamic_array_for_each (self,
                                 FOREACHFUNC (_private_for_each_item_free),
                                 NULL);
@@ -100,6 +101,14 @@
 
 spointer
 s_dynamic_array_get (SDynamicArray * self, size_t index) {
+  if (index > self->max_size) {
+    s_err_print ("Index out of range:\n\tMax index: %zd requested index:%zd",
+                 self->max_size, index);
+    return NULL;
+  }
+  if (self->max_size < 1) {
+    return NULL;
+  }
   return self->array[index];
 }
 
@@ -159,9 +168,11 @@
 void
 s_dynamic_array_for_each (SDynamicArray * self, ForEachFunc func,
                              spointer data) {
+  s_dbg_print ("Size: %zd", self->max_size);
   for (int i = 0; i < self->max_size && i <= self->last_item; i++) {
     spointer item = self->array[i];
     if (item) {
+      s_dbg_print ("Item Found.");
       func (self, item, data);
     }
   }

=== modified file 'libssts/Func.h'
--- libssts/Func.h	2015-10-01 10:46:50 +0000
+++ libssts/Func.h	2015-10-28 14:45:22 +0000
@@ -34,7 +34,7 @@
 /** @file
  * Generic function pointer definitions and data stuctures to do with
  * functions.
- * 
+ *
  * @todo create an object type that can hold functions and be able to run them.
  */
 
@@ -51,6 +51,8 @@
 #define FREEFUNC(k) ((FreeFunc) k)
 #define CALLBACK(k) ((Callback) k)
 #define FOREACHFUNC(k) ((ForEachFunc) k)
+#define COMPFUNC(k) ((CompFunc) k)
+
 
 /*
  * Common types of function pointers:
@@ -67,9 +69,10 @@
 /**
  * A generic callback function.
  *
- * A callback function can have any signature.
+ * @param obj The object to perform the callback on.
+ * @param user_data The user data to be passed to the function.
  */
-typedef void (* Callback)(void);
+typedef spointer (* Callback)(spointer obj, spointer user_data);
 
 /**
  *
@@ -92,10 +95,10 @@
 
 /**
  * A CompFunc represents a standard comparison function.
- * 
+ *
  * @param 1 object to compare
  * @param 2 object to compare
- * 
+ *
  * compare parameter 1 with parameter 2.
  */
 typedef sboolean (* CompFunc)(void*, void*);

=== modified file 'libssts/LinkedList.c'
--- libssts/LinkedList.c	2015-10-26 18:59:48 +0000
+++ libssts/LinkedList.c	2015-10-28 14:45:22 +0000
@@ -67,6 +67,7 @@
 void
 s_linked_list_free (SLinkedList * self, sboolean free_data) {
   //First we free the items if such is specified.
+  s_dbg_print ("Freeing SLinkedList");
   if (free_data) {
     s_dbg_print ("Freeing Data in LinkedList");
     s_linked_list_for_each (self,
@@ -91,7 +92,9 @@
 internal_s_linked_list_initialize_first_item (SLinkedList * self) {
   assert (self->len == 0);
   assert (self->head == NULL && self->tail == NULL && self->current == NULL);
-  self->head = self->tail = self->current = malloc (sizeof (SLinkedListNode));
+  self->current = malloc (sizeof (SLinkedListNode));
+  self->head = self->current;
+  self->tail = self->current;
   self->head->prev = NULL;
   self->tail->next = NULL;
 }
@@ -148,7 +151,10 @@
 
 sboolean
 s_linked_list_next (SLinkedList * self) {
-  if (self->current->next) {
+  if (self->current == NULL) {
+    return FALSE;
+  }
+  if (self->current->next != NULL) {
     self->current = self->current->next;
     return TRUE;
   } else {
@@ -255,6 +261,7 @@
 
   if(self->current != NULL) {
     do {
+      s_dbg_print ("s_map_linked_list_for_each: doing loop.");
       spointer cur_itm = s_linked_list_get_current (self);
       if (cur_itm != NULL) {
         func (self, cur_itm, user_data);

=== modified file 'libssts/Map.c'
--- libssts/Map.c	2015-10-24 19:55:57 +0000
+++ libssts/Map.c	2015-10-28 14:45:22 +0000
@@ -31,7 +31,7 @@
                           * in turn also SDynamicArrays; or in template speak:
                           * SDynamicArray<SLinkedList<SMapItem*>*>*
                           */
-  
+
   CompFunc is_equal; /*  method to check if items are equal. */
   HashFunc key_hash_func;
   FreeFunc free_key;
@@ -41,7 +41,7 @@
 void
 _s_map_internal_free_map_items_for_each (SMap * self,
                                          SMapItem * item,
-                                         sboolean * free_data);
+                                         sboolean free_data);
 
 SLinkedList *
 _s_map_internal_find_bucket (SMap * self,
@@ -57,7 +57,7 @@
   SMapItem * self = malloc (sizeof (SMapItem));
   self->key = key;
   self->value = value;
-  
+
   return self;
 }
 
@@ -65,49 +65,47 @@
 s_map_item_free (SMapItem * self,
                  FreeFunc free_key,
                  FreeFunc free_value) {
-  free_key (self->key);
-  free_value (self->value);
+  s_dbg_print ("Freeing Item!");
+  if (free_key) {
+    free_key (self->key);
+  }
+  if (free_value) {
+    free_value (self->value);
+  }
   free (self);
 }
 
-
+void
+_internal_s_map_free_buckets (SLinkedList * self);
 
 SMap *
 s_map_new (CompFunc comp_func,
            HashFunc key_hash_func,
            FreeFunc free_key,
            FreeFunc free_value) {
+  s_dbg_print ("Freeing SMap");
   SMap * self = malloc (sizeof (SMap));
   self->len = 0;
-  
+
   self->is_equal = comp_func;
-  
+
   /* free_* functions need to be checked if they are null and set the pointer
    * to free or s_base_object_free ()... Have to decide which...?
    */
-  if (free_key) {
-    self->free_key = free_key;
-  } else {
-    self->free_key = FREEFUNC(free);
-  }
-  if (free_value) {
-    self->free_value = free_value;
-  } else {
-    self->free_value = FREEFUNC(free);
-  }
-  
-  
-  /* if no func is set we have to use some other metod of 
+  self->free_key = free_key;
+  self->free_value = free_value;
+
+  /* if no func is set we have to use some other metod of
    */
   if (key_hash_func == NULL){
     self->key_hash_func = s_hash_object;
   } else {
     self->key_hash_func = key_hash_func;
   }
-  
+
   self->array = s_dynamic_array_new (S_MAP_DEFAULT_NUMBER_OF_BUCKETS,
-                                     FREEFUNC(s_linked_list_free));
-  
+                                     FREEFUNC(_internal_s_map_free_buckets));
+
   return self;
 }
 
@@ -124,33 +122,37 @@
 s_map_add (SMap * self, spointer key, spointer value) {
   SDynamicArray * array = self->array;
   SMapItem * item = s_map_item_new (key, value);
-  
+
   /* We have to generate a key to use as an index in the DynamicArray. */
   HashFunc hash_func = self->key_hash_func;
   hash_t hash = hash_func (key);
   /* We need to mod it with the max size of the array. */
   hash = hash % S_MAP_DEFAULT_NUMBER_OF_BUCKETS;
-  
+
   if (self->len == 0) {
     /* Since we know that there are no items in the array we can skip the
      * check if the new place is taken. and just and an array to it.
      */
-    SLinkedList * bucket = s_linked_list_new (NULL);
+    s_dbg_print ("Adding bucket to bucket array");
+    SLinkedList * bucket = s_linked_list_new (FREEFUNC(_s_map_internal_free_map_items_for_each));
+    s_dbg_print ("1) Appending item to linked list");
     s_dynamic_array_set (array, hash, bucket);
     s_linked_list_append (bucket, item);
-    
   } else {
+    s_dbg_print ("SMap size in more than zero.");
     /* Figure out if the bucket exists. */
     SLinkedList * bucket = _s_map_internal_find_bucket (self, key);
-    
+
     if (bucket == NULL) {
-      bucket = s_linked_list_new (NULL);
+      s_dbg_print ("Bucket does not exist");
+      bucket = s_linked_list_new (FREEFUNC(_s_map_internal_free_map_items_for_each));
     }
-    
+
     if (_s_map_internal_find_item_in_bucket (self, bucket, key)) {
       s_warn_print ("Key already exists in SMap\n");
       return;
     }
+    s_dbg_print ("2) Appending item to linked list");
     s_linked_list_append (bucket, item);
   }
   self->len++;
@@ -159,7 +161,7 @@
 spointer
 s_map_get (SMap * self, spointer key) {
   spointer ret_val = NULL;
-  
+
   SLinkedList * bucket = _s_map_internal_find_bucket (self, key);
   if (bucket) {
     SMapItem * item = _s_map_internal_find_item_in_bucket (self, bucket, key);
@@ -167,7 +169,7 @@
       ret_val = item->value;
     }
   }
-  
+
   return ret_val;
 }
 
@@ -189,10 +191,13 @@
 
 void
 s_map_for_each (SMap * self, ForEachFunc foreach_func, spointer user_data) {
+  s_dbg_print ("Running For Each on SMap");
   for (size_t i = 0; i <= s_dynamic_array_last_item (self->array); i++) {
     SLinkedList * bucket = s_dynamic_array_get (self->array, i);
     if (bucket) {
+      s_dbg_print ("Found bucket");
       s_linked_list_head (bucket);
+      s_dbg_print ("No of items in list: %zd.", s_linked_list_len (bucket));
       do {
         SMapItem * item = s_linked_list_get_current (bucket);
         foreach_func (self, item, user_data);
@@ -204,13 +209,14 @@
 SLinkedList *
 _s_map_internal_find_bucket (SMap * self,
                              spointer key) {
+  s_dbg_print ("Looking for bucket");
   SLinkedList * ret_val = NULL;
-  
+
   HashFunc hash_func = self->key_hash_func;
-  hash_t hash = hash_func (key);
-  
+  hash_t hash = hash_func (key)  % S_MAP_DEFAULT_NUMBER_OF_BUCKETS;
+
   ret_val = s_dynamic_array_get (self->array, hash);
-  
+
   return ret_val;
 }
 
@@ -219,14 +225,17 @@
 _s_map_internal_find_item_in_bucket (SMap * self,
                                      SLinkedList * bucket,
                                      spointer key) {
+  s_dbg_print ("Looking for item in bucket");
   s_linked_list_head (bucket);
   do {
     SMapItem * item = SMAPITEM (s_linked_list_get_current (bucket));
+
     if (item && self->is_equal (item->key, key)) {
+      s_dbg_print ("Found Item in Bucket.");
       return item;
     }
   } while (s_linked_list_next (bucket));
-  
+
   return NULL;
 }
 
@@ -245,10 +254,18 @@
 void
 _s_map_internal_free_map_items_for_each (SMap * map,
                           SMapItem * item,
-                          sboolean * free_data) {
-  if ((*free_data)) {
+                          sboolean free_data) {
+  s_dbg_print ("runnnig: _s_map_internal_free_map_items_for_each");
+  if (free_data) {
+    s_dbg_print ("Freeing K:V pair");
     s_map_item_free (item, map->free_key, map->free_value);
   } else {
     free (item);
   }
 }
+
+void
+_internal_s_map_free_buckets (SLinkedList * self) {
+  s_dbg_print ("Freeing bucket.");
+  s_linked_list_free (self, TRUE);
+}

=== modified file 'libssts/Thread.c'
--- libssts/Thread.c	2015-10-01 10:46:50 +0000
+++ libssts/Thread.c	2015-10-28 14:45:22 +0000
@@ -1,21 +1,5 @@
 
 #include "Thread.h"
-#if 0
-  // We get this from src/external/threads.h now.
-#if __WIN32__ || __WIN64__
-  #if (NTDDI_VERSION >= NTDDI_WIN8)
-    #include <Synchapi.h>
-  #else
-    #include <WinBase.h>
-  #endif
-#else
-  /* To get nanosleep () we must define __USE_POSIX199309 for som reason.
-   */
-  #define __USE_POSIX199309
-  #include <time.h>
-  #undef __USE_POSIX199309
-#endif
-#endif
 
 /*
   Utility functions associated with threads.
@@ -27,7 +11,7 @@
   Sleep (us / 1000)
 #else
   /* We are not a windows system, so lets assume that we have posix's nanosleep
-   * Taken from: 
+   * Taken from:
    */
   struct timespec req = {0};
   req.tv_sec = (int)(us / 1000);
@@ -50,11 +34,11 @@
 SMutex *
 s_mutex_new () {
   SMutex * self = malloc (sizeof (SMutex));
-  
+
   atomic_init(&(self->locked), FALSE);
-  
+
   mtx_init (&(self->mutex), mtx_plain);
-  
+
   return self;
 }
 
@@ -95,18 +79,18 @@
 struct SThread {
   thrd_t thread;
   RunFunc func;
-  
+
   _Atomic(sboolean) is_running;
 };
 
 SThread *
 s_thread_new (RunFunc func) {
   SThread * self = malloc (sizeof (SThread));
-  
+
   self->func = func;
-  
+
   atomic_store (&(self->is_running), FALSE);
-  
+
   return self;
 }
 

=== modified file 'libssts/baseobject.c'
--- libssts/baseobject.c	2015-10-26 16:45:37 +0000
+++ libssts/baseobject.c	2015-10-28 14:45:22 +0000
@@ -97,6 +97,11 @@
 
   self->refcount = 1;
 
+  self->callbacks = s_map_new (COMPFUNC (s_string_is_equal),
+                               HASH_FUNC (sdbm_hash),
+                               FREEFUNC (free),
+                               FREEFUNC (s_callback_entry_free));
+
   s_object_set_free_method (self, method_base_free);
   s_object_set_ref_method (self, method_base_ref);
   s_object_set_unref_method (self, method_base_unref);
@@ -119,6 +124,7 @@
 void
 s_object_free (SObject * self) {
   free (self->name);
+  s_map_free (self->callbacks, TRUE);
   SObjectClass * klass = s_object_get_class (self);
   klass->free (self);
 }
@@ -193,7 +199,8 @@
 
 char *
 method_base_to_string (SObject * self) {
-  char * ret_string = s_string_new_fmt ("(SObject, References: %d)", self->base_class->get_refcount(self));
+  char * ret_string = s_string_new_fmt ("(SObject, References: %d)",
+                                        self->base_class->get_refcount(self));
   return ret_string;
 }
 

=== modified file 'tests/CMakeLists.txt'
--- tests/CMakeLists.txt	2015-04-10 21:36:10 +0000
+++ tests/CMakeLists.txt	2015-10-28 14:45:22 +0000
@@ -9,6 +9,10 @@
   to_string.c
   dynamic_array.h
   dynamic_array.c
+  callback.h
+  callback.c
+  thread.h
+  thread.c
 )
 
 add_executable (ssts_test ${TEST_FILES})

=== added file 'tests/callback.c'
--- tests/callback.c	1970-01-01 00:00:00 +0000
+++ tests/callback.c	2015-10-28 14:45:22 +0000
@@ -0,0 +1,44 @@
+#include "../libssts/SimpleTypeSystem.h"
+#include "callback.h"
+#include "test_macros.h"
+
+int
+test_callback_test_func1 (SObject * obj, spointer user_data) {
+  return 2;
+}
+
+spointer
+test_callback_test_func2 (SObject * obj, spointer user_data) {
+
+  return NULL;
+}
+
+enum {
+  TEST_CALLBACK1,
+  TEST_CALLBACK2,
+  TEST_CALLBACK_END
+};
+
+static SCallbackEntry * callback_entries[] = {NULL,};
+
+int test_callback (void) {
+  setup_unit ();
+
+  SObject * obj = s_object_new ();
+
+  callback_entries[TEST_CALLBACK1] =
+    s_callback_entry_new ("test1",
+                          test_callback_test_func1,
+                          S_CALLBACK_CALLBACK);
+
+  callback_entries[TEST_CALLBACK2] =
+    s_callback_entry_new ("test2",
+                          test_callback_test_func2,
+                          S_CALLBACK_CALLBACK);
+
+  s_object_install_callbacks (obj, TEST_CALLBACK_END ,callback_entries);
+
+  s_object_free (obj);
+
+  end_unit ();
+}

=== added file 'tests/callback.h'
--- tests/callback.h	1970-01-01 00:00:00 +0000
+++ tests/callback.h	2015-10-28 14:45:22 +0000
@@ -0,0 +1,4 @@
+#ifndef __H_TEST_CALLBACK__
+#define __H_TEST_CALLBACK__
+int test_callback (void);
+#endif

=== modified file 'tests/main.c'
--- tests/main.c	2015-04-10 21:36:10 +0000
+++ tests/main.c	2015-10-28 14:45:22 +0000
@@ -9,15 +9,18 @@
 #include "to_string.h"
 #include "error.h"
 #include "dynamic_array.h"
+#include "callback.h"
 
 
 int main  (int argc, char ** argv, char ** argp) {
-  setup_suite("SSTS");
-  
+  setup_suite ("SSTS");
+
   test_unit (test_refcount, "Refcount");
   test_unit (test_to_string, "To String");
   test_unit (test_error, "Error");
   test_unit (test_dynamic_array, "Dynamic Array");
-  
-  end_suite();
+  test_unit (test_callback, "Callback");
+  test_unit (test_thread, "Threading");
+
+  end_suite ();
 }

=== added file 'tests/thread.c'
--- tests/thread.c	1970-01-01 00:00:00 +0000
+++ tests/thread.c	2015-10-28 14:45:22 +0000
@@ -0,0 +1,13 @@
+#include "test_macros.h"
+#include "thread.h"
+
+
+
+int
+test_thread (void) {
+  setup_unit ()
+
+
+
+  end_unit ()
+}

=== added file 'tests/thread.h'
--- tests/thread.h	1970-01-01 00:00:00 +0000
+++ tests/thread.h	2015-10-28 14:45:22 +0000
@@ -0,0 +1,4 @@
+#ifndef __H_TEST_THREAD__
+#define __H_TEST_THREAD__
+int test_thread (void);
+#endif /*__H_TEST_THREAD__*/

